# -*- coding: utf-8 -*-
# Elisa - Home multimedia server
# Copyright (C) 2006-2008 Fluendo Embedded S.L. (www.fluendo.com).
# All rights reserved.
#
# This file is available under one of two license agreements.
#
# This file is licensed under the GPL version 3.
# See "LICENSE.GPL" in the root of this distribution including a special
# exception to use Elisa with Fluendo's plugins.
#
# The GPL part of Elisa is also available under a commercial licensing
# agreement from Fluendo.
# See "LICENSE.Elisa" in the root directory of this distribution package
# for details on that license.

from pgm.timing.controller import INFINITE
from pgm.timing import implicit
import pgm

from elisa.plugins.pigment.graph.node import Node
from elisa.plugins.pigment.graph.image import Image
from elisa.plugins.pigment.widgets.theme import Theme

from elisa.core.common import application

import math


class LoadableMixin(Node):
    """
    Add loading animation feedback functionality to widgets.
    """

    loading_attribute = None

    def __init__(self, *args, **kwargs):
        super(LoadableMixin, self).__init__(*args, **kwargs)

        # FIXME: we need the frontend to use cloned images (by
        # "load_from_theme). This a cheap - UGLY - way to get the frontend
        # without changing much client code.
        self.frontend = application.interface_controller.frontends['frontend1']

        self._icon = None
        self._animated_icon = None

        self._interaction = None
        self._animated_interaction = None

        self._allow_animation = True
        self._loading = False

    def activate(self, previous=None):
        if self.loading_attribute:
            self._icon = getattr(self, self.loading_attribute, None)

        if not self._icon:
            return

        if previous and previous != self:
            previous.toggle_loading_animation(False)

        if previous != self:
            self._allow_animation = True
            if self._animated_icon is not None:
                self._animated_icon.stop_animations()
            self._animated_icon = implicit.AnimatedObject(self._icon)
            settings = {'end_callback' : lambda ctrl: self.toggle_loading_animation(True),
                        'duration' : 400}
            self._animated_icon.setup_next_animations(**settings)
            self._animated_icon.opacity = 0

            if self._interaction is not None:
                self._interaction.clean()
            self._loading = False

            self._create_interaction()

    def _create_interaction(self):
        self._interaction = Image()
        self.add(self._interaction)
        self._interaction.bg_color = (0, 0, 0, 0)
        self._interaction.opacity = 255
        self.frontend.load_from_theme("elisa.plugins.poblesec.waiting_icon", self._interaction)
        self._interaction.set_name('wait icon')

        if self.loading_attribute:
            self._icon = getattr(self, self.loading_attribute, None)

        if self._icon:
            self._interaction.size = self._icon.size
            self._interaction.position = self._icon.position

        self._animated_interaction = implicit.AnimatedObject(self._interaction,
                                                             ('rz', 'opacity'))

    def toggle_loading_animation(self, enabled):
        if enabled and self._allow_animation:
            if self._loading:
                return

            if not self._interaction:
                self._create_interaction()

            self._loading = True
            self._animated_interaction.setup_next_animations('rz',
                                                 repeat_count=INFINITE,
                                                 duration=960)
            self._animated_interaction.setup_next_animations('opacity',
                                                 duration=390)
            self._interaction.rz = 0.0
            self._interaction.opacity = 10
            self._animated_interaction.rz = math.pi * 2
            self._interaction.visible = True
            self._animated_interaction.opacity = 255
        else:
            if not self._interaction:
                return

            self._allow_animation = False
            self._loading = False
            self._interaction.clear()
            self._interaction.visible = False
            self._animated_interaction.stop_animations()

            settings = {'end_callback' : None,
                        'duration' : 400}

            if self._icon:
                self._animated_icon.setup_next_animations(**settings)
                self._animated_icon.opacity = 255

    def clean(self):
        if self._interaction is not None:
            self._interaction.clean()
            self._interaction = None

        if self._animated_interaction is not None:
            self._animated_interaction.stop_animations()
            self._animated_interaction = None

        if self._animated_icon is not None:
            self._animated_icon.stop_animations()
            self._animated_icon = None

        return super(LoadableMixin, self).clean()

